SystemOrganization addCategory: #'Seaside-RenderLoop'!

!WAPackage class methodsFor: '*seaside-renderloop' stamp: 'jok 11/28/2008 15:55'!
seasideRenderLoop
	^ self new
		name: 'Seaside-RenderLoop';
		description: 'Defines the low-level rendering and callback processing functionality.';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Component';
		yourself! !

!WASession methodsFor: '*seaside-renderloop' stamp: 'jf 10/17/2008 20:44'!
inNewRenderLoopCall: aComponent
	^ WAContinuation currentDo: [ :cc | 
		aComponent onAnswer: cc.
		self inNewRenderLoopShow: aComponent ] ! !

!WASession methodsFor: '*seaside-renderloop' stamp: 'jf 10/19/2008 14:28'!
inNewRenderLoopShow: aComponent
	((self preferenceAt: #renderPhaseContinuationClass) root: aComponent) value! !

WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopConfiguration commentStamp: 'jf 7/22/2008 12:32' prior: 0!
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 10/17/2008 20:59'!
continuationClasses
	^ WARenderLoopContinuation allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 1/2/2009 20:55'!
describeOn: config
	(config list: #actionPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s action phase.';
		options: [ self continuationClasses ];
		default: WAActionPhaseContinuation.
	(config list: #renderPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s render phase.';
		options: [ self continuationClasses ];
		default: WARenderPhaseContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [ self rootClasses ].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [ self rootDecorationClasses ].
	config
		at: #mainClass put: WARenderLoopMain! !

!WARenderLoopConfiguration methodsFor: 'ancestry' stamp: 'jf 7/24/2008 19:10'!
parents
	"We override several attributes of the session configuration."
	^ Array with: WASessionConfiguration instance! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 7/29/2008 14:30'!
rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 12:58'!
rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

WAError subclass: #WAMissingResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAMissingResponseError methodsFor: 'accessing' stamp: 'pmm 11/4/2008 20:47'!
possibleCauses
	^ #(
		'A Component attempted to use #call: or a similar method while rendering. Calling and answering can only be done in a callback.'
	)! !

WASessionContinuation subclass: #WARenderLoopContinuation
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

WARenderLoopContinuation subclass: #WAActionPhaseContinuation
	instanceVariableNames: 'renderingContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 1/18/2009 20:04'!
handleRequest
	| renderContinuation |
	[ self withNotificationHandlerDo: [ 
		self renderingContext callbacks handle: self requestContext ] ]
			ensure: ["this code is not yet tested because the javascript code isn't working with the response refactoring  yet. but I think it will work. we might want to copy states before snapshotting it -- jf'" 
				self request isXmlHttpRequest ifTrue: [ self states snapshot ] ].
	renderContinuation := self createRenderContinuation.
	(self shouldRedirect) 
		ifTrue: [ self redirectToContinuation: renderContinuation ]
		ifFalse: [ renderContinuation value ]! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 9/28/2008 23:37'!
renderingContext
	^ renderingContext! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 9/28/2008 23:37'!
renderingContext: anObject 
	renderingContext := anObject! !

!WAActionPhaseContinuation methodsFor: 'private' stamp: 'jf 10/19/2008 12:41'!
shouldRedirect
	| request |
	request := self request.
	^ (request isGet and: [ request fields includesKey: '_n' ]) not! !

!WARenderLoopContinuation class methodsFor: 'instance creation' stamp: 'jf 9/28/2008 23:02'!
root: aComponent
	^ self new initializeWithRoot: aComponent! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 1/18/2009 19:18'!
createActionContinuation
	"Answer a new action phase continuation."

	^ ((self session preferenceAt: #actionPhaseContinuationClass) root: self root)
			setStates: self states;
			yourself! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 1/18/2009 19:19'!
createRenderContinuation
	"Answer a new render phase continuation."

	^ (self session preferenceAt: #renderPhaseContinuationClass) root: self root! !

!WARenderLoopContinuation methodsFor: 'initialization' stamp: 'jf 1/17/2009 12:17'!
initializeWithRoot: aComponent
	self initialize.
	root := aComponent! !

!WARenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 1/18/2009 20:02'!
root
	"Answer the root component instance. Usually this component is instantiated with the first request."

	^ root! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 1/18/2009 23:38'!
updateRoot: aHtmlRoot
	aHtmlRoot bodyAttributes at: 'onload' append: 'onLoad()' separator: ';'.
	super updateRoot: aHtmlRoot.
	(WAVisiblePresenterGuide client:
		(WAUpdateRootVisitor root: aHtmlRoot))
		start: self root! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 11/3/2008 16:13'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	(WAAllPresenterGuide client:
		(WAUpdateStatesVisitor snapshot: aSnapshot))
		start: self root! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 11/3/2008 16:09'!
updateUrl: anUrl
	super updateUrl: anUrl.
	(WAVisiblePresenterGuide client: (WAUpdateUrlVisitor url: anUrl))
		start: self root! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 11/3/2008 08:25'!
withNotificationHandlerDo: aBlock
	^ aBlock on: WARenderNotification do: [ :n | ]! !

WARenderLoopContinuation subclass: #WARenderPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/18/2009 19:34'!
createDocumentFor: aResponse 
	^  self application newDocumentOn: aResponse stream! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/18/2009 23:37'!
createHtmlRootWithContext: aRenderinContext
	| htmlRoot |
	htmlRoot := WAHtmlRoot context: aRenderinContext.
	self updateRoot: htmlRoot.
	^ htmlRoot! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/18/2009 19:36'!
createRenderingContext
	| renderingContext |
	renderingContext := WARenderingContext new.
	
	"Wrap the default visitor with a rendering guide"
	renderingContext visitor: (WARenderingGuide client: renderingContext visitor).
	
	^ renderingContext
		! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 1/18/2009 20:00'!
handleRequest
	"We no longer really need to catch the notification handler here but
	we need #call: to generate a useful error response during the render
	phase so, until we improve that, it's still here.'"
	self withNotificationHandlerDo: [
		self session clearJumpTo.
		self respond: [ :response | self processRendering: response ] ].

	"The render phase should have returned a response"
	WAMissingResponseError signal! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 1/18/2009 23:38'!
processRendering: aResponse 
	| document htmlRoot renderingContext actionContinuation |
	"Create objects we need"
	document := self createDocumentFor: aResponse.
	actionContinuation := self createActionContinuation.
	renderingContext := self createRenderingContext.
	
	"Hook objects together"
	renderingContext
		document: document;
		actionUrl: actionContinuation registerForUrl.
	actionContinuation renderingContext: renderingContext.
	htmlRoot := self createHtmlRootWithContext: renderingContext.

	"Render"
	document open: htmlRoot.
	self root renderWithContext: renderingContext.
	self writeOnLoadOn: document.
	document close: htmlRoot.
	renderingContext release! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 9/28/2008 23:29'!
writeOnLoadOn: aDocument 
	| attributes |
	attributes := WAHtmlAttributes new.
	attributes 
		at: 'type'
		put: 'text/javascript'.
	aDocument 
		openTag: 'script'
		attributes: attributes
		closed: false.
	aDocument nextPutAll: '/*<!![CDATA[*/function onLoad(){'.
	self requestContext loadScripts 
		do: [ :each | aDocument nextPutAll: each seasideString ]
		separatedBy: [ aDocument nextPutAll: ';' ].
	aDocument nextPutAll: '}/*]]>*/'.
	aDocument closeTag: 'script'! !

WAMain subclass: #WARenderLoopMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopMain commentStamp: 'rew 10/19/2007 10:50' prior: 0!
When a new session on a WAApplication is started WARenderLoopMain initializes the application, that is it:
 
	creates the top level component of the application, 
	informs each component(WAPresenter) of the application that the session started (via WAPresenter>>initialRequest:)
	starts a WARenderLoop to handle the request
!

!WARenderLoopMain methodsFor: 'creational' stamp: 'lr 9/14/2008 13:14'!
createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 12:56'!
rootClass
	^ self application preferenceAt: #rootClass! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 13:03'!
rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses! !

!WARenderLoopMain methodsFor: 'processing' stamp: 'jf 11/3/2008 16:20'!
start
	| root |
	root := self createRoot.
	(WAAllPresenterGuide client:
		(WAInitialRequestVisitor request: self requestContext request))
		start: root.
	((self session preferenceAt: #renderPhaseContinuationClass) root: root) value! !
